@@ -87,16 +87,17 @@ PipeOpCopy = R6Class("PipeOpCopy",
       assert_int(outnum, lower = 1)
       super$initialize(id, param_vals = param_vals,
         input = data.table(name = "input", train = "*", predict = "*"),
-        output = data.table(name = rep_suffix("output", outnum), train = "*", predict = "*")
+        output = data.table(name = rep_suffix("output", outnum), train = "*", predict = "*"),
+        tags = "meta"
       )
-    },
-
-    train_internal = function(inputs) {
+    }
+  ),
+  private = list(
+    .train = function(inputs) {
       self$state = list()
       rep_len(inputs, self$outnum)
     },
-
-    predict_internal = function(inputs) {
+    .predict = function(inputs) {
       rep_len(inputs, self$outnum)
     }
   )
