@@ -1,6 +1,5 @@
 context("PipeOpFeatureUnion")
 
-
 test_that("featureunion - basic properties", {
   po = PipeOpFeatureUnion$new(3)
   expect_pipeop(po)
@@ -15,14 +14,16 @@ test_that("featureunion - basic properties", {
   expect_data_table(po$input, nrows = 1)
   expect_data_table(po$output, nrows = 1)
 
+  expect_error(PipeOpFeatureUnion$new(1, collect_multiplicity = TRUE), regexp = "collect_multiplicity only works with innum == 0")
 })
 
-
 test_that("PipeOpFeatureUnion - train and predict", {
   # Define PipeOp's
   tsk = mlr_tasks$get("iris")
-  t1 = tsk$clone()$set_col_role(c("Sepal.Length", "Sepal.Width"), character())
-  t2 = tsk$clone()$set_col_role(c("Petal.Length", "Petal.Width"), character())
+  keep1 = setdiff(tsk$feature_names, c("Sepal.Length", "Sepal.Width"))
+  keep2 = setdiff(tsk$feature_names, c("Petal.Length", "Petal.Width"))
+  t1 = tsk$clone()$select(keep1)
+  t2 = tsk$clone()$select(keep2)
 
   expect_task(cbind_tasks(inputs = list(t1, t2), TRUE, c("", "")))
 
@@ -45,11 +46,9 @@ test_that("PipeOpFeatureUnion - train and predict", {
   pout = predict_pipeop(po, list(t1, t2))
   expect_equivalent(pout[[1]]$feature_names, tsk$feature_names)
   expect_equivalent(pout[[1]]$target_names, tsk$target_names)
-
 })
 
 test_that("PipeOpFeatureUnion - train and predict II", {
-
   # Define PipeOp's
   scatter = PipeOpCopy$new(2)
   op2a = PipeOpPCA$new()
@@ -57,12 +56,11 @@ test_that("PipeOpFeatureUnion - train and predict II", {
   op3 = PipeOpFeatureUnion$new(2)
   opdot = PipeOpFeatureUnion$new()
 
-
   task = mlr_tasks$get("iris")
   lrn = mlr_learners$get("classif.rpart")
   op4 = PipeOpLearner$new(learner = lrn)
 
-  #  FIXME: Check param_set
+  # FIXME: Check param_set
   param_names_union = c(
     "OpNULL.Petal.Width", "OpNULL.Petal.Length",
     "pca.PC1", "OpNULL.Sepal.Length", "pca.PC2",
@@ -93,23 +91,25 @@ test_that("PipeOpFeatureUnion - train and predict II", {
   expect_set_equal(trained$feature_names, c("PC1", "PC2", "PC3", "PC4", "Petal.Length",
     "Petal.Width", "Sepal.Length", "Sepal.Width"))
   expect_true(graph$is_trained)
-
 })
 
 test_that("Test wrong inputs", {
   # Differing rows
   pos = PipeOpSubsample$new()
   pos$param_set$values$frac = 0.5
-  g = greplicate(
+  g = pipeline_greplicate(
     pos %>>% PipeOpPCA$new(),
     2
-  ) %>>% PipeOpFeatureUnion$new(2)
+  ) %>>% PipeOpFeatureUnion$new(c("a", "b"))
   task = mlr_tasks$get("iris")
   expect_error(g$train(task), "Assertion on 'rows'")
+
+  # Differing target columns
+  po = PipeOpFeatureUnion$new()
+  expect_error(po$train(list(tsk("iris"), tsk("mtcars")), regexp = "All tasks must have the same target columns"))
 })
 
 test_that("PipeOpFeatureUnion - levels are preserved", {
-
   tbl1 = data.table(x1 = factor(letters[10:14], levels = letters), y1 = letters[1:5], target = 1:5)
   tbl2 = data.table(x2 = factor(letters[10:14], levels = letters), y2 = letters[1:5], target = 1:5)
 
@@ -120,17 +120,14 @@ test_that("PipeOpFeatureUnion - levels are preserved", {
   tsk2$col_info
 
   pofu = PipeOpFeatureUnion$new(2)
-
+  expect_true(!pofu$is_trained)
   pofu$train(list(tsk1, tsk2))[[1]]$col_info
-
-
+  expect_true(pofu$is_trained)
   pofu$train(list(tsk1$filter(3:5), tsk2$filter(3:5)))[[1]]$col_info
-
-
+  expect_true(pofu$is_trained)
 })
 
 test_that("feature renaming", {
-
   expect_pipeop_class(PipeOpFeatureUnion, list(letters[1:3]))
 
   expect_equal(nrow(PipeOpFeatureUnion$new(c("a", "b", "c"))$input), 3)
@@ -158,7 +155,7 @@ test_that("feature renaming", {
   lrn = mlr_learners$get("classif.rpart")
   op4 = PipeOpLearner$new(learner = lrn)
 
-  #  FIXME: Check param_set
+  # FIXME: Check param_set
   param_names_union = c(
     "OpNULL.Petal.Width", "OpNULL.Petal.Length",
     "pca.PC1", "OpNULL.Sepal.Length", "pca.PC2",
@@ -179,10 +176,8 @@ test_that("feature renaming", {
 
   po = PipeOpFeatureUnion$new(c("z", "a", "a"))
 
-# FIXME: this needs https://github.com/mlr-org/mlr3/issues/268
-#  expect_equal(po$train(list(task, task, PipeOpPCA$new()$train(list(task))[[1]]))[[1]]$feature_names,
-#    c(task$feature_names, paste0("a.", task$feature_names), paste0("a.PC", 1:4)))
-
+  expect_equal(po$train(list(task, task, PipeOpPCA$new()$train(list(task))[[1]]))[[1]]$feature_names,
+    c(task$feature_names, paste0("a.", task$feature_names), paste0("a.PC", 1:4)))
 })
 
 # https://github.com/mlr-org/mlr3pipelines/issues/216
@@ -202,3 +197,43 @@ test_that("feature renaming", {
 ##
 ##   pofu$train(list(t1, t2))
 ## })
+
+test_that("featureunion - duplicates in feature names", {
+  tsk = mlr_tasks$get("iris")
+
+  g = pipeline_greplicate(PipeOpPCA$new(), 2) %>>% PipeOpFeatureUnion$new(2)
+
+  # this should work (just keeps each PC a single time)
+  train_out_g = g$train(tsk)
+
+  popca = PipeOpPCA$new()
+  train_out_pca = popca$train(list(tsk))
+  expect_equal(train_out_g[[1]]$data(), train_out_pca[[1]]$data())
+
+  # the following should not (duplicated feature names but actually different values)
+  dat = tsk$data()
+  dat$Petal.Length = rnorm(150)
+  dat$Sepal.Width[1] = 999
+  tsk2 = TaskClassif$new("tsk2", backend = dat, target = "Species")
+
+  po = PipeOpFeatureUnion$new()
+  expect_error(po$train(list(tsk, tsk2)), regexp = "different features sharing the same feature name")
+})
+
+test_that("featureunion - collect_multiplicity", {
+  po = PipeOpFeatureUnion$new(0, collect_multiplicity = TRUE)
+  expect_pipeop(po)
+  expect_data_table(po$input, nrows = 1)
+  expect_data_table(po$output, nrows = 1)
+
+  tsk = mlr_tasks$get("iris")
+  keep1 = setdiff(tsk$feature_names, c("Sepal.Length", "Sepal.Width"))
+  keep2 = setdiff(tsk$feature_names, c("Petal.Length", "Petal.Width"))
+  t1 = tsk$clone()$select(keep1)
+  t2 = tsk$clone()$select(keep2)
+
+  train_out = po$train(list(as.Multiplicity(list(t1, t2))))
+  predict_out = po$predict(list(as.Multiplicity(list(t1, t2))))
+  expect_equal(train_out[[1]]$data, tsk$data)
+  expect_equal(predict_out[[1]]$data, tsk$data)
+})
